@@ -2,7 +2,7 @@
 
   class.c -
 
-  $Author: shugo $
+  $Author: yugui $
   created at: Tue Aug 10 15:05:44 JST 1993
 
   Copyright (C) 1993-2007 Yukihiro Matsumoto
@@ -10,10 +10,8 @@
 **********************************************************************/
 
 #include "ruby/ruby.h"
-#include "ruby/signal.h"
-#include "ruby/node.h"
 #include "ruby/st.h"
-#include "vm_core.h"
+#include "node.h"
 #include <ctype.h>
 
 extern st_table *rb_class_tbl;
@@ -72,6 +70,8 @@ struct clone_method_data {
     VALUE klass;
 };
 
+VALUE rb_iseq_clone(VALUE iseqval, VALUE newcbase);
+
 static int
 clone_method(ID mid, NODE *body, struct clone_method_data *data)
 {
@@ -190,7 +190,25 @@ VALUE
 rb_make_metaclass(VALUE obj, VALUE super)
 {
     if (BUILTIN_TYPE(obj) == T_CLASS && FL_TEST(obj, FL_SINGLETON)) {
-	return RBASIC(obj)->klass = rb_cClass;
+        VALUE metaclass, meta_of_super;
+        if (RBASIC(obj)->klass == obj) { /* for meta^(n)-class of Class */
+            metaclass = rb_class_boot(obj);
+            RBASIC(metaclass)->klass = metaclass;
+        }
+        else {
+            metaclass = rb_class_boot(super);
+            RBASIC(metaclass)->klass = rb_singleton_class(RBASIC(obj)->klass);
+        }
+        FL_SET(metaclass, FL_SINGLETON);
+        rb_singleton_class_attached(metaclass, obj);
+        RBASIC(obj)->klass = metaclass;
+
+        meta_of_super = RCLASS(obj)->ptr->super;
+        while (FL_TEST(meta_of_super, T_ICLASS)) {
+            meta_of_super = RCLASS(meta_of_super)->ptr->super;
+        }
+        RCLASS(metaclass)->ptr->super = rb_singleton_class(meta_of_super);
+        return metaclass;
     }
     else {
 	VALUE metasuper;
@@ -819,7 +837,6 @@ rb_singleton_class(VALUE obj)
 	rb_bug("unknown immediate %ld", obj);
     }
 
-    DEFER_INTS;
     if (FL_TEST(RBASIC(obj)->klass, FL_SINGLETON) &&
 	rb_iv_get(RBASIC(obj)->klass, "__attached__") == obj) {
 	klass = RBASIC(obj)->klass;
@@ -840,7 +857,6 @@ rb_singleton_class(VALUE obj)
 	FL_UNSET(klass, FL_UNTRUSTED);
     }
     if (OBJ_FROZEN(obj)) OBJ_FREEZE(klass);
-    ALLOW_INTS;
 
     return klass;
 }
