@@ -2,8 +2,7 @@
 
   sockport.h -
 
-  $Author: shyouhei $
-  $Date: 2007-02-12 16:01:19 -0700 (Mon, 12 Feb 2007) $
+  $Author: akr $
   created at: Fri Apr 30 23:19:34 JST 1999
 
 ************************************************/
