@@ -3,6 +3,15 @@ require 'rational'
 require 'timeout'
 
 class TestTime < Test::Unit::TestCase
+  def setup
+    @verbose = $VERBOSE
+    $VERBOSE = nil
+  end
+
+  def teardown
+    $VERBOSE = @verbose
+  end
+
   def test_time_add()
     assert_equal(Time.utc(2000, 3, 21, 3, 30) + 3 * 3600,
                  Time.utc(2000, 3, 21, 6, 30))
@@ -373,12 +382,14 @@ class TestTime < Test::Unit::TestCase
     assert_equal("2000", T2000.strftime("%Y"))
     assert_equal("UTC", T2000.strftime("%Z"))
     assert_equal("%", T2000.strftime("%%"))
+    assert_equal("0", T2000.strftime("%-S"))
 
     assert_equal("", T2000.strftime(""))
     assert_equal("foo\0bar\x0000\x0000\x0000", T2000.strftime("foo\0bar\0%H\0%M\0%S"))
     assert_equal("foo" * 1000, T2000.strftime("foo" * 1000))
 
-    assert_equal("Sat", Time.at(946684800).strftime("%a"))
+    t = Time.mktime(2000, 1, 1)
+    assert_equal("Sat", t.strftime("%a"))
 
     t = Time.at(946684800, 123456.789)
     assert_equal("123", t.strftime("%3N"))
@@ -387,8 +398,51 @@ class TestTime < Test::Unit::TestCase
     assert_equal("1234567890", t.strftime("%10N"))
     assert_equal("", t.strftime("%0N"))
     assert_equal("000", t.strftime("%3S"))
+    assert_equal("946684800", t.strftime("%s"))
+    assert_equal("946684800", t.utc.strftime("%s"))
 
     t = Time.mktime(2001, 10, 1)
     assert_equal("2001-10-01", t.strftime("%F"))
+
+    t = Time.mktime(2001, 10, 1, 2, 0, 0)
+    assert_equal("01", t.strftime("%d"))
+    assert_equal("01", t.strftime("%0d"))
+    assert_equal(" 1", t.strftime("%_d"))
+    assert_equal(" 1", t.strftime("%e"))
+    assert_equal("01", t.strftime("%0e"))
+    assert_equal(" 1", t.strftime("%_e"))
+    assert_equal("AM", t.strftime("%p"))
+    assert_equal("am", t.strftime("%#p"))
+    assert_equal("am", t.strftime("%P"))
+    assert_equal("AM", t.strftime("%#P"))
+    assert_equal("02", t.strftime("%H"))
+    assert_equal("02", t.strftime("%0H"))
+    assert_equal(" 2", t.strftime("%_H"))
+    assert_equal("02", t.strftime("%I"))
+    assert_equal("02", t.strftime("%0I"))
+    assert_equal(" 2", t.strftime("%_I"))
+    assert_equal(" 2", t.strftime("%k"))
+    assert_equal("02", t.strftime("%0k"))
+    assert_equal(" 2", t.strftime("%_k"))
+    assert_equal(" 2", t.strftime("%l"))
+    assert_equal("02", t.strftime("%0l"))
+    assert_equal(" 2", t.strftime("%_l"))
+    t = Time.mktime(2001, 10, 1, 14, 0, 0)
+    assert_equal("PM", t.strftime("%p"))
+    assert_equal("pm", t.strftime("%#p"))
+    assert_equal("pm", t.strftime("%P"))
+    assert_equal("PM", t.strftime("%#P"))
+    assert_equal("14", t.strftime("%H"))
+    assert_equal("14", t.strftime("%0H"))
+    assert_equal("14", t.strftime("%_H"))
+    assert_equal("02", t.strftime("%I"))
+    assert_equal("02", t.strftime("%0I"))
+    assert_equal(" 2", t.strftime("%_I"))
+    assert_equal("14", t.strftime("%k"))
+    assert_equal("14", t.strftime("%0k"))
+    assert_equal("14", t.strftime("%_k"))
+    assert_equal(" 2", t.strftime("%l"))
+    assert_equal("02", t.strftime("%0l"))
+    assert_equal(" 2", t.strftime("%_l"))
   end
 end
